//import your libraries. 
import de.bezier.data.sql.*;

MySQL msql;
ArrayList transactionArray;
ArrayList itemtypeArray;
PFont font1 = createFont("Helvetica", 16); //"Helvetica" the documentary <-- netflix 

int barWidth = 10;
int barHeight = 20;
int verticalSpacing = 5;
int leftBorder = 10; //how much empty space you want on either side of your graph (in pixels)

void setup() {
  size(900, 640); //size of the sketch window
  colorMode(HSB, 360,100,100); //color mode- like RGB or HSB (learn to use HSB because it's the best: http://en.wikipedia.org/wiki/HSL_and_HSV)
  transactionArray = new ArrayList();  // Create an empty ArrayList
  itemtypeArray = new ArrayList();
  textAlign(LEFT,CENTER);
  smooth();
  
    // connecting to the KRAKEN database
    msql = new MySQL( this, "kraken.mat.ucsb.edu", "spl0", "josh", "sneezes with wolves" ); //host, database, user, password
    if ( msql.connect() )
    { 
      String TheQuery="select date(cout),date(cin),itemtype,timestampdiff(HOUR,cout,cin)/24.0 from inraw where cout between '2011-01-10' and '2011-01-11' order by itemtype;";
          println(TheQuery);
          msql.query( TheQuery);
          msql.next();
          int count = 0;
          while( msql.next() ){
              //println( msql.getInt(1) );
              transactionArray.add(new Transaction(msql.getString(1),msql.getString(2),msql.getString(3),msql.getFloat(4)));
              count++;
          }     
     }
    else
    {
      println(" connection failed !");
    }
    
    //populate the itemtypeArray with all the *unique* itemtypes and their count
    for (int i = 0; i < transactionArray.size(); i++) { 
    // An ArrayList doesn't know what it is storing so we have to cast the object coming out
    Transaction transaction = (Transaction) transactionArray.get(i);
      boolean iFoundTheWord = false;
      // Is the word already in the Array?
      for(int j = 0;j<itemtypeArray.size();j++){
        TransactionGroup tg = (TransactionGroup) itemtypeArray.get(j);
        if(tg.name.equals(transaction.itemtype)){
          tg.count++;
          transaction.itemtypeIndex = j;
          tg.averageDuration += transaction.durationInDays;
          iFoundTheWord = true;
        }
      }
      if(iFoundTheWord == false){
        //also need to add duration into the word constructor now that we're finding the averages. 
        itemtypeArray.add(new TransactionGroup(transaction.itemtype, transaction.durationInDays));
        transaction.itemtypeIndex = itemtypeArray.size()-1;
      }
    }    
    println("how many unique item types: " + itemtypeArray.size());
      
    //after you find all the unique item types and their count, we want to find the average duration of transaction for each item type.    
    for(int i = 0;i<itemtypeArray.size();i++){
      TransactionGroup tg = (TransactionGroup) itemtypeArray.get(i);
      tg.averageDuration = tg.averageDuration/tg.count;
      //now that we have an average, we'll use it to calculate an x and y position so that we don't have to do it later.  
      tg.xPosition = 50+leftBorder+tg.averageDuration*3.5;
      tg.yPosition = 40+(0.5*barHeight)+((verticalSpacing+barHeight)*i);
    }  
    
    //so that we don't have to do a million complicated calculations during the draw loop, we're going to do a few right here
    for (int i = 0; i < transactionArray.size(); i++) { 
      // An ArrayList doesn't know what it is storing so we have to cast the object coming out
      Transaction transaction = (Transaction) transactionArray.get(i);
      //calculate the x and y location of your line
      transaction.xPosition = 50+leftBorder+transaction.durationInDays*3.5;
      transaction.yPosition = 40+((barHeight+verticalSpacing)*transaction.itemtypeIndex);
    }  
}

void draw() {
    background(360);
    textAlign(LEFT,CENTER);
    
    //draw a small line for each individual transaction
    for (int i = 0; i < transactionArray.size(); i++) { 
      Transaction transaction = (Transaction) transactionArray.get(i);
      stroke(0,0,0);
      strokeWeight(0.07);
      line(transaction.xPosition, transaction.yPosition, transaction.xPosition, transaction.yPosition + barHeight); //draw the line
      noFill();
    }
    
    //draw all of the item type labels ("acbk", "acdvd", etc.)
    for(int i = 0;i<itemtypeArray.size();i++){
      TransactionGroup tg = (TransactionGroup) itemtypeArray.get(i);
      fill(0,0,0);
      text(tg.name, leftBorder,40+(0.5*barHeight)+((verticalSpacing+barHeight)*i));
    }
    text("days", leftBorder,40+(0.5*barHeight)+((verticalSpacing+barHeight)*itemtypeArray.size()));
    
    //draw the horizontal labels and grid lines
    for(int i = 0;i<8;i++){
      float tempX = 50+leftBorder+i*30*3.5;
      float tempY = 40+(0.5*barHeight)+((verticalSpacing+barHeight)*itemtypeArray.size());
      fill(0,0,0);      
      textAlign(CENTER,CENTER);
      text(i*30, tempX,tempY);
      stroke(0,0,50);
      strokeWeight(0.5);
      line(tempX,tempY-10,tempX,40);
    }
    
    //draw the red averages line
    for(int i = 1;i<itemtypeArray.size();i++){
      TransactionGroup tg = (TransactionGroup) itemtypeArray.get(i-1);
      TransactionGroup tg2 = (TransactionGroup) itemtypeArray.get(i); 
      strokeWeight(1.4);
      stroke(0,80,70); 
      line(tg.xPosition,tg.yPosition,tg2.xPosition,tg2.yPosition);    
    }      
   
    /*strokeWeight(1);
    stroke(0,0,0);
    line(mouseX,height,mouseX,0);
    text((mouseX-50-leftBorder)/22.5,mouseX,mouseY);*/  
      
    //give it a title  
    fill(0,0,0);  
    textAlign(LEFT,CENTER);
    text("Duration of Checkouts from the Seattle Public Library on January 10, 2011 categorized by item type.",50+leftBorder, 20); 
    
    fill(0,80,70);
    text("Averages plotted in red.",700, 20);
    
    fill(0,0,0);  
    textAlign(CENTER,CENTER);
    text("length of checkout in days", width/2,40+(0.5*barHeight)+((verticalSpacing+barHeight)*(itemtypeArray.size()+1)));
    
}
